mod v1;
mod v2;

use std::fs;

use anyhow::Result;
pub use v2::Project;

use crate::dirs;


#[derive(Debug, Default, serde::Serialize, serde::Deserialize)]
pub struct Projects {
    list: Vec<Project>,
}

impl Projects {
    pub fn iter(&self) -> std::slice::Iter<'_, Project> {
        self.list.iter()
    }

    pub fn list(&self) -> &[Project] {
        &self.list
    }

    pub fn remove(&mut self, idx: usize) -> Project {
        self.list.remove(idx)
    }

    pub fn add(&mut self, project: Project) {
        self.list.push(project);
    }

    pub fn get_mut(&mut self, idx: usize) -> Option<&mut Project> {
        self.list.get_mut(idx)
    }

    pub fn read_file() -> Result<Projects> {
        let projects_file = dirs::data_path().join("projects");

        if !projects_file.exists() {
            return Ok(Projects::default());
        }

        let list = match v2::deserialize_project_list(&projects_file) {
            Ok(list) => list,
            Err(_) => v1::deserialize_project_list(&projects_file)?,
        };

        Ok(Projects { list })
    }

    pub fn write_file(&self) -> Result<()> {
        let projects_file = dirs::data_path().join("projects");

        let file = fs::OpenOptions::new()
            .write(true)
            .create(true)
            .truncate(true)
            .open(projects_file)?;

        bincode::serialize_into(file, &self.list)?;

        Ok(())
    }
}


impl<'a> IntoIterator for &'a Projects {
    type IntoIter = std::slice::Iter<'a, Project>;
    type Item = &'a Project;

    fn into_iter(self) -> Self::IntoIter {
        self.list.iter()
    }
}
