use std::path::{Path, PathBuf};

use anyhow::Result;
use tracing::trace;


#[derive(Debug, Clone, PartialEq, serde::Serialize, serde::Deserialize)]
pub struct Project {
    pub path: PathBuf,
}

impl From<PathBuf> for Project {
    fn from(path: PathBuf) -> Self {
        Self { path }
    }
}


#[tracing::instrument(skip_all)]
pub fn deserialize_project_list<P>(path: P) -> Result<Vec<super::Project>>
where
    P: AsRef<Path>,
{
    let file = std::fs::File::open(path)?;
    trace!("Deserializing and migrating V1 project list");

    let list = bincode::deserialize_from::<_, Vec<Project>>(file)
        .map(|projects| projects.into_iter().map(From::from).collect())?;

    Ok(list)
}
